# This file is a portion of the Red Hat Network Panel Applet
#
# Copyright (C) 1999-2002 Red Hat, Inc. All Rights Reserved.
# Distributed under GPL version 2.
#
# Author: Chip Turner
#
# $Id: rhn_applet_model.py,v 1.28 2005/12/01 03:45:55 rnorwood Exp $

import string
import os
import time

import gettext
import rhn_utils

from rhn_applet_rpm import rhnAppletRPM
from rhn_applet_rpc import rhnAppletRPC
from rhn_applet_yum import rhnAppletYum
from rhn_applet_apt import rhnAppletApt

from rhn_utils import rhnAppletException
from rhn_utils import rhnAppletNagNotConfiguredException
from rhn_utils import rhnAppletRPCFault
from rhn_sources import getSources, changedSources

try:
    from rhn import rpclib
    from rhn import xmlrpclib
except ImportError:
    rpclib = __import__("xmlrpclib")

GETTEXT_DOMAIN = "rhn-applet"
gettext.bindtextdomain (GETTEXT_DOMAIN, "/usr/share/locale")
gettext.textdomain (GETTEXT_DOMAIN)
try:
    gettext.install(GETTEXT_DOMAIN, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode
except AttributeError: # older gettext modules don't have an install() method
    from gettext import _

class rhnAppletModel:
    def __init__(self, refresh_callback, url, uuid, mode = "gui", cert = None):
        self.__refresh_callback__ = refresh_callback
        self.__url__ = url
        self.__uuid__ = uuid

        self.__proxy__ = ''
        self.__proxy_username__ = ''
        self.__proxy_password__ = ''
        
        self.__arch__ = os.uname()[4]

        self.__needed_packages__ = []
        self.__ignored_packages__ = {}
        self.__release__ = None
        self.__kernel_running__ = None
	self.__sources__ = None
	self.__sources_list__ = getSources()

        self.__rpmdb__ = None

        self.__last_rpm_change_number__ = 0
        self.__last_sources_change_number__ = 0

        # this depends on server caps
        self.use_has_base_channel = 0
        
        # public data member
        self.change_number = 0

        if not mode in ('gui', 'tui'):
            raise _("invalid rhnAppletModel mode %s") % mode
            
        self.__mode__ = mode
        self.__cert__ = cert

        # fetch server cap info, and populate
#        self.__get_caps()

    def __build_sources(self):
        if self.__sources_list__ == None:
	    self.__sources_list__ = getSources()
	if self.__sources_list__ == None:
	    self.__sources__ = None
	    return
	self.__sources__ = []
	has_up2date = 0
	for source in self.__sources_list__:
	    try:
		if source['type'] == 'up2date':
		    if has_up2date == 0:
			rpc_server = rhnAppletRPC(self.__url__,
                                               self.__refresh_callback__,
                                               self.__uuid__,
                                               self.__release__,
                                               self.__arch__,
                                               self.__cert__,
                                               self.__proxy__,
                                               self.__proxy_username__,
                                               self.__proxy_password__)
			if rpc_server != None:
			    self.__sources__.append(rpc_server)
			    has_up2date = 1
		elif source['type'] == 'yum':
		    src = rhnAppletYum(source['url'],
		                       self.__refresh_callback__,
		                       source['label'],
		                       self.__release__, self.__arch__,
				       self.__proxy__, self.__proxy_username__,
				       self.__proxy_password__)
		    if src != None:
		        rhn_utils.log_debug("adding yum source %s" %
			                    (source['label']))
		        self.__sources__.append(src)
		elif source['type'] == 'apt':
		    src = rhnAppletApt(source['url'],
		                       self.__refresh_callback__,
		                       source['label'],
		                       source['dist'],
		                       self.__release__, self.__arch__,
				       self.__proxy__, self.__proxy_username__,
				       self.__proxy_password__)
		    if src != None:
		        rhn_utils.log_debug("adding apt source %s" %
			                    (source['label']))
		        self.__sources__.append(src)
		else:
		    rhn_utils.log_debug("Unsupported source type %s for %s" %
		             (source['type'], source['url']))
	    except:
	        import sys
	        rhn_utils.log_debug("Failed to initialize source %s for %s : %s %s" % (
		           source['type'], source['url'],
			   sys.exc_type, sys.exc_value))
		        

    #
    # Depending on the set of source used the usage of the applet may or
    # may not require the user to consent the RHN licence usage. This
    # routine walks the list of sources to check if any such consent is
    # actually needed.
    #
    def need_consent(self):
        try:
	    if self.__sources_list__ == None:
		self.__sources_list__ = getSources()
	    if self.__sources_list__ == None:
		return 0
	    for source in self.__sources_list__:
		if source['type'] == 'up2date':
		    rhn_utils.log_debug("Applet uses RHN as source")
		    return 1
	except:
	    rhn_utils.log_debug("Exception checking if RHN is in sources")
	    return 0
	rhn_utils.log_debug("Applet does not uses RHN as source")
	return 0
        

    def set_proxy(self, proxy, u, p):
        self.__proxy__ = proxy
        self.__proxy_username__ = u
        self.__proxy_password__ = p

	if self.__sources__ != None:
	    for source in self.__sources__:
	        source.set_proxy(proxy, u, p)

    def is_package_ignored(self, name):
        if self.__ignored_packages__.has_key(name):
            return 1
        else:
            return 0
    def add_ignored_package(self, name):
        self.__ignored_packages__[name] = 1
    def remove_ignored_package(self, name):
        if self.__ignored_packages__.has_key(name):
            del self.__ignored_packages__[name]
    def ignored_package_list(self):
        return self.__ignored_packages__.keys()

    def installed_package(self, name):
        rpm = self.__rpmdb__

        return rpm.find_latest_by_name(name)

    def needed_packages(self):
        return self.__needed_packages__, self.__ignored_packages__.keys()

    def alerts(self):
        if not self.fully_initialized():
            return []

        ret = []
        # find the latest version of the package that has the same
        # name as the package that the currently running kernel
        # belongs to

        if len(self.__kernel_running__):
            latest_kernel = self.__rpmdb__.find_latest_by_name(self.__kernel_running__["name"])
            if not latest_kernel["epoch"]:
                latest_kernel["epoch"] = None

            if latest_kernel != self.__kernel_running__:
                a = rhnAppletAlert('kernel', self.__kernel_running__, latest_kernel)
                ret.append(a)
        else:
            # not running a kernel in RPM format?  not much we can do
            a = rhnAppletAlert('unsupported-kernel', os.uname()[2])
            ret.append(a)
            pass

        needed_up2date = None
        for pkg in self.__needed_packages__:
            if pkg["name"] != "up2date":
                continue

            needed_up2date = pkg
            break
        
        if needed_up2date:
            a = rhnAppletAlert('up2date', self.installed_package("up2date"), needed_up2date)
            ret.append(a)

        sources_errors = ""
	for source in self.__sources__:
	    if source.in_error():
	        if sources_errors == "":
		    sources_errors = source.name()
		else:
		    sources_errors = sources_errors + ", %s" %(source.name())
	if sources_errors != "":
	    a = rhnAppletAlert('source-error', sources_errors)
            ret.append(a)
        return ret

    # given the capabilties the server provides, do whatever is
    # neccasary to work around missing caps, etc. In most cases, this
    # means setting some state variable on the model, and branch code
    # based on that
    def __handle_caps(self, caps):
        # see if we have the server side for querying about
        # channel info sans system id
        if caps.has_key("applet.has_base_channel"):
            self.use_has_base_channel = 1

    # call the sources get_caps method, and try to deal with them
    def get_caps(self):

        # FIXME: this seems like the wrong place to do this, but
        # it works for now
        self.fully_initialized()
        if self.__sources__ != None:
            for source in self.__sources__:
                try:
                    caps = source.get_caps()
                    self.__handle_caps(caps)
                except:
                    import sys
                    rhn_utils.log_debug("failed to get server capabilty info: %s %s" %
                                        (sys.exc_type, sys.exc_value));
    
    #
    # Check for upgrade packages from a source
    #
    def __check_source_refresh(self, install, avail):
        rpm = self.__rpmdb__
        A = avail
        # format of elements of A is dict with keys:
        #   nvre, name, version, release, epoch, errata_advisory, errata_id

        I = install
        # I is a hash of installed packages.  keys are package
        # names, values is LATEST version of package as a dict:
        #   nvre, name, version, version, epoch

        for latest in A:
            if not I.has_key(latest['name']):
                # package not on this system?  next!
                continue

            installed = I[latest["name"]]

            cmp = rpm.compare_vre(latest["version"],
                                  latest["release"],
                                  latest["epoch"],
                                  installed["version"],
                                  installed["release"],
                                  installed["epoch"])

            if cmp <= 0:
                # we have the same or a more recent
                continue

            self.__needed_packages__.append(latest)

    # if we're not initialized, checkin simply returns, letting
    # defaults control behavior
    
    def refresh(self, force=0):
        if not self.fully_initialized():
            return

	if changedSources():
	    self.__sources__ = None
	    self.__sources_list__ = None
	    self.__build_sources()

        rpm = self.__rpmdb__
        installed = rpm.latest_installed_packages()
        self.__needed_packages__ = []

	if self.__sources__ != None:
	    for source in self.__sources__:
	        try:
		    avail = source.latest_packages()
		    self.__check_source_refresh(installed, avail)
		except:
		    import sys
		    rhn_utils.log_debug("failed to check source: %s %s" %
		                        (sys.exc_type, sys.exc_value));

        self.change_number = self.change_number + 1

    def has_base_channel(self):
        base_channel = None
        # if we get this far, assume the server supports this call
        # the caps based logic should be higher up
        if self.__sources__ != None:
            for source in self.__sources__:
                try:
                    base_channel = source.has_base_channel()
                except rhnAppletRPCFault, f:
                    raise
                except:
                    import sys
                    rhn_utils.log_debug("failed to check for proper system entitlement: %s %s" %
		                        (sys.exc_type, sys.exc_value));

        return base_channel
                    
        
    def fully_initialized(self):
        rpm = self.__rpmdb__
        
        if not self.__release__:
            releases = rpm.find_provides("redhat-release")
            if releases and len(releases):
                self.__release__ = releases[0]["version"]

        if self.__kernel_running__ == None:
            kernel_image = "/boot/vmlinuz-" + os.uname()[2]
            kernel_running = rpm.find_provides_by_file(kernel_image)

            if kernel_running and len(kernel_running):
                self.__kernel_running__ = {}
                for i in ("name", "version", "release", "epoch"):
                    self.__kernel_running__[i] = kernel_running[0][i]
            else:
                # no idea what kernel you're running
                rhn_utils.log_debug("could not figure out kernel")
                self.__kernel_running__ = [ ]

        if self.__release__ and not self.__sources__:
	    self.__build_sources()

        if self.__kernel_running__ != None and self.__sources__ and rpm.latest_installed_packages():
            return 1

        return 0

    # change numbers are monotonically increasing values; idea
    # borrowed from the database world.  if something changed, we mark
    # it.  higher level components can watch for changes and act
    # accordingly.  fits in well with GUIs so we don't redraw too
    # often.
    
    def poll_change_numbers(self):
        changed = 0
        
        if self.__last_rpm_change_number__ != self.__rpmdb__.change_number:
            self.__last_rpm_change_number__ = self.__rpmdb__.change_number
            self.change_number = self.change_number + 1
            changed = 1
            
	if changedSources():
	    changed = 1

	if self.__sources__ != None:
	    val = 0
	    for source in self.__sources__:
	        val = val + source.change_number()
	    if val != self.__last_sources_change_number__:
	        self.__last_sources_change_number__ = val;
	        
	        

        return changed

    def pkg_as_nvr(self, pkg):
        return string.join((pkg["name"], pkg["version"], pkg["release"]), "-")

    # The base model class will not do a nag check
    def nag_check(self):
        return 0

class rhnAppletModelTUI(rhnAppletModel):
    def __init__(self, refresh_callback, url, uuid, cert = None):
        rhnAppletModel.__init__(self, refresh_callback, url, uuid, 'tui', cert)

        self.__rpmdb__ = rhnAppletRPM(fatal = 1)
        self.__rpmdb__.refresh()

        while not self.fully_initialized():
            pass

	if self.__sources__ != None:
	    for source in self.__sources__:
	        if source != None:
		    source.refresh(1)

        self.change_number = 1
        self.poll_change_numbers()
        rhnAppletModel.refresh(self)
        
class rhnAppletModelGUI(rhnAppletModel):
    def __init__(self, refresh_callback, url, uuid, cert = None):
        rhnAppletModel.__init__(self, refresh_callback, url, uuid, 'gui', cert)
        self.__rpmdb__ = rhnAppletRPM(fatal = 0)

    def refresh(self, force=0):
        newconf = 0
        self.__rpmdb__.refresh(force)

        if not self.fully_initialized():
            return
        
	if changedSources():
	    #
	    # the sources configuration changed, update the server list
	    #
	    newconf = 1

	if self.__sources__ != None:
	    for source in self.__sources__:
	        if source != None:
		    source.refresh(force)

        if self.poll_change_numbers() or newconf == 1:
            rhnAppletModel.refresh(self)

    # Shall we nag, or not?  1, or 0.
    def nag_check(self):
        if not os.access(self.systemid_file, os.F_OK):
            # firstboot writes out the system nag file, as root.
            # we do this since firstboot is ignorant of what user account
            # might actually be running the applet to get the nag.
            # once we determine we need to nag, we keep the info about
            # when to nag each user in their home dir. The system nagfile
            # is to determine if we should nag users at all. 
            #
            # rhn_register will get changed to remove the system nag file
            # on registration
            if not os.access(self.system_nag_file, os.R_OK):
                raise rhnAppletNagNotConfiguredException("System nag file not found")

            if self.ready_to_nag():
                return 1
                
            # theres a system id, but we have to verify they actually have channels and
            # whatnot as well
        elif self.ready_to_nag():
            # if the server doesnt support this, we cant really
            # nag because of that.  In fact, do not nag any more.
            self.get_caps()
            if not self.use_has_base_channel:
                raise rhnAppletNagNotConfiguredException("Server does not support the applet.has_base_channel API call")

            # if they have a base channel, dont nag them
            try:
                bc = self.has_base_channel()
            except rhn_utils.rhnAppletRPCFault, e:
                print _("Error: %s" % e.fault_string)
                bc = None
                    
            if bc == None:
                # there was an error of some sort, probably perms
                # on the up2date-uuid file, dont nag in this case
                return 0
                
            if int(bc) != 0:
                raise rhnAppletNagNotConfiguredException("System is fully registered.  No need to nag.")

            return 1

        return 0

    def write_nag_timestamp(self):
        rhn_utils.set_user_config("remindTimeStamp", "%s" % time.time())

    def ready_to_nag(self):
        if rhn_utils.get_user_config("remindTimeStamp") == "NEVERREMIND":
            raise rhnAppletNagNotConfiguredException("The user asked not to be nagged")
        
        elif rhn_utils.get_user_config("remindTimeStamp") != None:
            if time.time() > (float(rhn_utils.get_user_config("remindTimeStamp")) + self.nag_interval):
                return 1
                # we havent setup a nag time yet
        if rhn_utils.get_user_config("remindTimeStamp") == None \
               or rhn_utils.get_user_config("remindTimeStamp") == "":
            # update the nag timestamp so we dont annoy again
            self.write_nag_timestamp()
            return 1

        return 0

class rhnAppletAlert:
    def __init__(self, type, *data):
        self.__type__ = type
        self.__data__ = data

    def is_unsupported_kernel_alert(self):
        if self.__type__ == 'unsupported-kernel':
            return 1
        return 0
    def is_kernel_alert(self):
        if self.__type__ == 'kernel':
            return 1
        return 0
    def is_up2date_alert(self):
        if self.__type__ == 'up2date':
            return 1
        return 0
    def is_source_alert(self):
        if self.__type__ == 'source-error':
            return 1
        return 0

    def alert_data(self):
        return self.__data__

